from distutils.core import setup

setup(
    name='Flask-Crud',
    packages=['flask_crud'],
    version='0.0.2',
    author='Andrew Plummer',
    author_email='plummer574@gmail.com',
    url='https://github.com/plumdog/flask_crud',
    description='Quick and Easy CRUD for Flask, built on top of Flask-Classy',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Operating System :: OS Independent',
        'Framework :: Flask',
    ])
